package com.ltdigital.sdk.android.demo;

import android.content.Context;
import android.content.SharedPreferences;

import com.google.gson.annotations.SerializedName;

/**
 * @author Riven
 * @version 1.0
 * @date 2020/3/17 13:55
 */
public class MyPreferences {

    private final static String PREFERENCES_FILE = "My.Preferences";

    //手表传输过来的步数今日显示缓存数据
    @SerializedName(value = "STORAGE_ACTIVATE_CODE")
    private static final String STORAGE_ACTIVATE_CODE = "Preferences.STORAGE_ACTIVATE_CODE";

    //手表传输过来的卡路里今日显示缓存数据
    @SerializedName(value = "STORAGE_APPID")
    private static final String STORAGE_APPID = "Preferences.STORAGE_APPID";

    //存储的默认激活服务器地址
    @SerializedName(value = "STORAGE_ACTIVE_SERVER_URL")
    private static final String STORAGE_ACTIVE_SERVER_URL = "Preferences.STORAGE_ACTIVE_SERVER_URL";

    //生产测试开关状态
    @SerializedName(value = "SWITCH_STATE_TEST")
    private static final String SWITCH_STATE_TEST = "Preferences.SWITCH_STATE_TEST";

    //激活设备开关状态
    @SerializedName(value = "SWITCH_STATE_VERIFY")
    private static final String SWITCH_STATE_VERIFY = "Preferences.SWITCH_STATE_VERIFY";

    private String UNKNOWN = "Unknown";

    private static MyPreferences dataPreferences;
    private SharedPreferences mSharedPreferences;

    public MyPreferences(Context context) {
        mSharedPreferences = context.getSharedPreferences(PREFERENCES_FILE, Context.MODE_PRIVATE);
    }

    public static MyPreferences getPreferences(Context context) {
        if (dataPreferences == null) {
            dataPreferences = new MyPreferences(context);
        }

        return dataPreferences;
    }

    public String getActivateCode() {
        return mSharedPreferences.getString(STORAGE_ACTIVATE_CODE, null);
    }

    public void setActivateCode(String activateCode) {
        mSharedPreferences.edit().putString(STORAGE_ACTIVATE_CODE, activateCode).apply();
    }

    public String getStorageAppId() {
        return mSharedPreferences.getString(STORAGE_APPID, null);
    }

    public void setStorageAppId(String appid) {
        mSharedPreferences.edit().putString(STORAGE_APPID, appid).apply();
    }

    public String getStorageActiveServerUrl() {
        return mSharedPreferences.getString(STORAGE_ACTIVE_SERVER_URL, "http://192.168.5.66:8080");
    }

    public void setStorageActiveServerUrl(String url) {
        mSharedPreferences.edit().putString(STORAGE_ACTIVE_SERVER_URL, url).apply();
    }

    public boolean getSwitchTestEnable() {
        return mSharedPreferences.getBoolean(SWITCH_STATE_TEST, true);
    }

    public void setSwitchTestEnable(boolean isEnable) {
        mSharedPreferences.edit().putBoolean(SWITCH_STATE_TEST, isEnable).apply();
    }

    public boolean getSwitchVerifyEnable() {
        return mSharedPreferences.getBoolean(SWITCH_STATE_VERIFY, true);
    }

    public void setSwitchVerifyEnable(boolean isEnable) {
        mSharedPreferences.edit().putBoolean(SWITCH_STATE_VERIFY, isEnable).apply();
    }

    public void clear() {
        mSharedPreferences.edit().clear().apply();
    }
}
