package com.ltdigital.sdk.android.demo.activity

import android.content.Context
import android.os.Handler
import android.os.Looper
import android.util.Log
import android.widget.Toast
import androidx.appcompat.app.AppCompatActivity
import com.android.core.base.mvp.inter.IView
import com.ltdigital.sdk.android.demo.dialog.LoadingDialog
import com.tbruyelle.rxpermissions3.RxPermissions

/**
 * @author  Riven
 * @date  2021/9/23 14:24
 * @version 1.0
 */
abstract class BaseActivity : AppCompatActivity(), IView {

    val TAG = BaseActivity::class.java.name
    var loadingDialog: LoadingDialog? = null
    val handler = Handler(Looper.getMainLooper())
    val rxPermission = RxPermissions(this)

    override fun onDestroy() {
        super.onDestroy()
        handler.removeCallbacksAndMessages(null)
    }

    override fun showToast(p0: CharSequence?) {
        runOnUiThread {
            Toast.makeText(applicationContext, p0, Toast.LENGTH_SHORT).show()
        }
    }

    override fun getContext(): Context {
        return this
    }

    override fun showError(p0: String?, p1: String?) {
        showToast("onError:$p0,$p1")
    }

    override fun startLoading() {
        startLoading(null)
    }

    protected fun startLoading(desc: String?) {
        Log.e("dialog", "startLoading")
        runOnUiThread {
            if (loadingDialog == null) {
                loadingDialog = LoadingDialog(this, this)
            }
            loadingDialog?.setDescText(desc)
            loadingDialog?.show()
        }
    }

    override fun finishLoading() {
        Log.e("dialog", "finishLoading")
        runOnUiThread {
            loadingDialog?.dismiss()
        }
    }
}