package com.ltdigital.sdk.android.demo.activity

import android.Manifest
import android.bluetooth.BluetoothManager
import android.bluetooth.le.ScanCallback
import android.bluetooth.le.ScanFilter
import android.bluetooth.le.ScanResult
import android.bluetooth.le.ScanSettings
import android.content.Intent
import android.os.Bundle
import android.view.View
import android.widget.AdapterView
import android.widget.ArrayAdapter
import android.widget.ListView
import android.widget.Toast
import androidx.appcompat.app.AppCompatActivity
import androidx.core.app.ActivityCompat
import androidx.core.content.PermissionChecker
import com.ltdigital.sdk.android.demo.R
import kotlinx.coroutines.GlobalScope
import kotlinx.coroutines.delay
import kotlinx.coroutines.launch

class ConnectActivity : AppCompatActivity() {
    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_connect)
        doScan()
        initList()
    }

    var scanList: ListView? = null
    var adapter: ArrayAdapter<String>? = null
    var deviceSet: HashSet<String>? = null
    var isScan = false
    private fun initList() {
        scanList = findViewById(R.id.scanList)
        adapter = ArrayAdapter(this, android.R.layout.simple_list_item_1, android.R.id.text1)
        deviceSet = hashSetOf()
        scanList?.adapter = adapter
        scanList?.onItemClickListener =
            AdapterView.OnItemClickListener { parent, view, position, id ->
                val mac = adapter?.getItem(position)
                val retList = mac?.split("\n")
                val data = Intent()
                data.putExtra("mac", retList?.get(retList.size - 1))
                setResult(RESULT_OK, data)
                finish()
            }
        findViewById<View>(R.id.button).setOnClickListener {
            doScan()
        }
    }

    val scanCallback = object : ScanCallback() {
        override fun onScanResult(callbackType: Int, result: ScanResult?) {
            if (result?.device?.name == null)
                return
            if (result.device.name.equals("WDC-SB").not())
                return
            result.device?.address?.let { mac ->
                deviceSet?.contains(mac)?.let {
                    if (it.not()) {
                        adapter?.add("${result.device?.name}_${result.rssi}\n${result.device?.address}")
                        deviceSet?.add(mac)
                    }
                }
            }
        }
    }

    private fun doScan() {
        if (isScan) return
        isScan = true
        adapter?.clear()
        deviceSet?.clear()
        val btManager = getSystemService(BLUETOOTH_SERVICE) as BluetoothManager
        if (!btManager.adapter.isEnabled)
            btManager.adapter.enable()

        btManager.adapter?.let {
            it.bluetoothLeScanner.startScan(
                arrayListOf(ScanFilter.Builder().build()),
                ScanSettings.Builder().setScanMode(ScanSettings.SCAN_MODE_LOW_LATENCY).build(),
                scanCallback
            )
            findViewById<View>(R.id.button).isEnabled = false
            stopScan().start()
        }
    }

    private fun stopScan() = GlobalScope.launch {
        if (!isScan) return@launch
        delay(10000)
        runOnUiThread {
            isScan = false
            val btManager = getSystemService(BLUETOOTH_SERVICE) as BluetoothManager
            btManager.adapter.bluetoothLeScanner.stopScan(scanCallback)
            findViewById<View>(R.id.button).isEnabled = true
        }
    }

    override fun onRequestPermissionsResult(
        requestCode: Int,
        permissions: Array<out String>,
        grantResults: IntArray,
    ) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults)
        for (r in grantResults) {
            if (r != PermissionChecker.PERMISSION_GRANTED) {
                Toast.makeText(this, "需要权限才能继续。", Toast.LENGTH_LONG).show()
                setResult(RESULT_CANCELED)
                finish()
                return
            }
        }
        doScan()
    }
}
