package com.ltdigital.sdk.android.demo.activity

import android.graphics.Color
import android.graphics.Typeface
import android.os.Bundle
import android.text.style.AbsoluteSizeSpan
import android.text.style.CharacterStyle
import android.text.style.ForegroundColorSpan
import android.text.style.StyleSpan
import android.util.Log
import android.view.MotionEvent
import android.view.View
import com.android.core.Utils
import com.android.core.adapter.BaseViewPagerAdapter
import com.android.core.base.fragment.BaseFragment
import com.ltdigital.sdk.android.demo.R
import com.ltdigital.sdk.android.demo.bean.AutoTestResult
import com.ltdigital.sdk.android.demo.bluetooth.Operation
import com.ltdigital.sdk.android.demo.callback.TestControlCallback
import com.ltdigital.sdk.android.demo.callback.TestResultCallback
import com.ltdigital.sdk.android.demo.fragment.AutoResultFragment
import kotlinx.android.synthetic.main.activity_device_test.*

/**
 * @author  Riven
 * @date  2021/11/16 18:47
 * @version 1.0
 */
class DeviceTestActivity : BaseDeviceActivity(), TestControlCallback,
    TestResultCallback {

    private var adapter: BaseViewPagerAdapter? = null
    private var leftTouchPass = false
    private var rightTouchPass = false
    private var needTestHeart = false

    private val autoFragment = AutoResultFragment.newInstance(
        "自动测试结果", this, AutoResultFragment.TAG_TEST_TYPE_AUTO
    )

    private val touchFragment = AutoResultFragment.newInstance(
        "触摸测试\n请同时触碰按键观察左右指示亮起后放开完成测试", this, AutoResultFragment.TAG_TEST_TYPE_TOUCH
    )

    private val ledVibratorFragment = AutoResultFragment.newInstance(
        "LED/马达测试\n观察震动及LED灯结果后选择测试结果", this, AutoResultFragment.TAG_TEST_TYPE_LED_VIBRATOR
    )

    private val heartFragment = AutoResultFragment.newInstance(
        "心率、血氧测试\n观察心率及血氧出值结果后选择测试结果", this, AutoResultFragment.TAG_TEST_TYPE_HEART_SPO2
    )

    private val chargeFragment = AutoResultFragment.newInstance(
        "充电测试\n接入充电线，监测到充电信号后观察电流选择充电结果", this, AutoResultFragment.TAG_TEST_TYPE_CHARGER
    )

    private val powerFragment = AutoResultFragment.newInstance(
        "功耗测试\n" + "观察功耗后选择测试结果", this, AutoResultFragment.TAG_TEST_TYPE_POWER
    )

    val timeoutRunnable = Runnable {
        showToast("获取自动测试结果超时!")
        dealFactoryTestCancel()
    }

    override fun initSuccess() {
        beginFactoryTest()
        MainActivity.bleManager?.setTestResultCallback(this)
    }

    override fun onDestroy() {
        super.onDestroy()
        MainActivity.bleManager?.setTestResultCallback(null)
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_device_test)
        initViewPager()
//        tv_test_log?.movementMethod = ScrollingMovementMethod.getInstance()
    }

    fun initViewPager() {
        val fragments = mutableListOf<BaseFragment>()
        fragments.add(
            autoFragment
        )
        fragments.add(
            touchFragment
        )
        fragments.add(
            ledVibratorFragment
        )
        fragments.add(
            heartFragment
        )
        fragments.add(
            chargeFragment
        )
        fragments.add(
            powerFragment
        )

        adapter = BaseViewPagerAdapter(supportFragmentManager, fragments)
        viewPager.adapter = adapter
        viewPager.offscreenPageLimit = fragments.size
        viewPager.setOnTouchListener(object : View.OnTouchListener {
            override fun onTouch(v: View?, event: MotionEvent?): Boolean {
                return true
            }
        })
    }

    private fun beginFactoryTest() {
        startLoading("正在获取自动测试结果...")
        handler.postDelayed(timeoutRunnable, 5 * 1000)
        MainActivity.bleManager?.sendTestCommand(arrayOf(0x01))
    }

    private fun printLog(msg: String) {
        Log.d(TAG, msg)
//        tv_test_log?.post {
//            tv_test_log?.let {
//                val format = SimpleDateFormat("HH:mm:ss.SSS", Locale.getDefault())
//                it.append(
//                    "${
//                        format.format(Date(System.currentTimeMillis()))
//                    } ->  $msg\n"
//                )
//                val offset = it.lineCount * it.lineHeight
//                val contentHeight = it.height - it.paddingTop - it.paddingBottom
//                if (offset > contentHeight) {
//                    it.scrollTo(0, offset - contentHeight)
//                }
//            }
//        }
    }

    override fun onPass(tag: String?) {
        printLog("onPass:$tag")
        var pageIndex = viewPager.currentItem
        MainActivity.bleManager?.sendTestCommand(arrayOf(0x03, 0x01))
        MainActivity.bleManager?.sendTestCommand(when (tag) {
            AutoResultFragment.TAG_TEST_TYPE_TOUCH -> {
                pageIndex = 2
                arrayOf(0x02, 0x02)
            }
            AutoResultFragment.TAG_TEST_TYPE_LED_VIBRATOR -> {
                if (needTestHeart) {
                    pageIndex = 3
                    arrayOf(0x02, 0x05)
                } else {
                    pageIndex = 4
                    arrayOf(0x02, 0x03)
                }
            }
            AutoResultFragment.TAG_TEST_TYPE_HEART_SPO2 -> {
                pageIndex = 4
                arrayOf(0x02, 0x03)
            }
            AutoResultFragment.TAG_TEST_TYPE_CHARGER -> {
                pageIndex = 5
                arrayOf(0x02, 0x04)
            }
            AutoResultFragment.TAG_TEST_TYPE_POWER -> {
                showToast("测试通过,已自动断开连接")
                arrayOf(0x05)
            }
            else -> {
                null
            }
        }, object : Operation.DataSyncOperationCallback {
            override fun onFail(status: Int) {
            }

            override fun onSuccess() {
                if (tag == AutoResultFragment.TAG_TEST_TYPE_POWER) {
                    dealFactoryTestFinish()
                }
                viewPager.setCurrentItem(pageIndex, true)
            }
        })
    }

    private fun dealFactoryTestFinish() {
        finishLoading()
        showToast("测试完成")
        setResult(RESULT_OK)
        finish()
    }

    private fun dealFactoryTestCancel() {
        finishLoading()
        showToast("测试未完成")
        setResult(RESULT_CANCELED)
        finish()
    }


    override fun onNG(tag: String?) {
        printLog("onNg:$tag")
        showToast("测试超时,设备主动断开")
        MainActivity.bleManager?.sendTestCommand(
            arrayOf(0x03, 0x00),
            object : Operation.DataSyncOperationCallback {
                override fun onFail(status: Int) {
                    dealFactoryTestCancel()
                }

                override fun onSuccess() {
                    dealFactoryTestCancel()
                }
            })
    }

    override fun onNext(tag: String?) {
        printLog("onNext:$tag")
        if (tag == AutoResultFragment.TAG_TEST_TYPE_AUTO) {
            MainActivity.bleManager?.sendTestCommand(arrayOf(0x02, 0x01))
            viewPager.setCurrentItem(1, true)
        }
    }

    override fun onReceivedAutoTestResult(result: AutoTestResult) {
        handler.removeCallbacks(timeoutRunnable)
        finishLoading()
        val pass = "PASS"
        val ng = "NG"

        needTestHeart = result.type != 0x00

        val resultText: Array<String> = arrayOf(
            "测试类型:",
            result.getTestType(),
            "\n六轴:",
            if (result.IMUPass) pass else ng,
            "\t\t地磁:",
            if (result.magPass) pass else ng,
            "\nNFC:",
            if (result.NfcPass) pass else ng,
            "\t\tNAND:",
            if (result.NANDPass) pass else ng,
            "\nUSB:",
            if (result.USBPass) pass else ng,
            "\t\t心率:",
            if (result.heartPass) pass else ng,
            "\n电池NTC:",
            if (result.batteryPass) pass else ng
        )

        val style: Array<Array<CharacterStyle>> = Array(resultText.size) {
            when (resultText[it]) {
                pass -> {
                    arrayOf(
                        ForegroundColorSpan(
                            Color.parseColor("#33CC33")
                        ), AbsoluteSizeSpan(Utils.sp2px(this, 20f))
                    )
                }
                ng -> {
                    arrayOf(
                        ForegroundColorSpan(
                            Color.parseColor("#FF3333")
                        ), AbsoluteSizeSpan(Utils.sp2px(this, 20f))
                    )
                }
                result.getTestType() -> {
                    arrayOf(
                        ForegroundColorSpan(
                            Color.parseColor("#0099FF")
                        ), AbsoluteSizeSpan(
                            Utils.sp2px(this, 20f)
                        ),
                        StyleSpan(Typeface.BOLD)
                    )
                }
                else -> {
                    arrayOf(
                        ForegroundColorSpan(
                            Color.BLACK
                        ), AbsoluteSizeSpan(Utils.sp2px(this, 20f))
                    )
                }
            }
        }

        autoFragment.setContent(
            Utils.getSubString(
                resultText, style
            )
        )
        autoFragment.setAutoTestIsPass(result.isAllPass())

    }

    override fun onReceivedTouchTestResult(leftPass: Boolean, rightPass: Boolean) {
        if (viewPager.currentItem == 1) {
            if (leftPass) {
                leftTouchPass = leftPass
            }
            if (rightPass) {
                rightTouchPass = rightPass
            }
        }
        touchFragment?.onTouchResult(leftPass, rightPass, leftTouchPass, rightTouchPass)
        if (viewPager.currentItem == 1 && leftTouchPass && rightTouchPass && !leftPass && !rightPass) {
            //触摸测试通过
            onPass(AutoResultFragment.TAG_TEST_TYPE_TOUCH)
        }
    }

    override fun onReceivedChargeTestResult(chargePass: Boolean) {
        chargeFragment?.onChargeSign()
    }

    override fun onReceivedHeartTestResult(heartValue: Int?, spo2Value: Int?) {
        heartFragment.setContent("hr:${heartValue ?: "--"}\nspo2:${spo2Value ?: "--"}")
    }

    override fun onReceivedTimeout() {
        onNG(null)
    }
}