package com.ltdigital.sdk.android.demo.activity

import android.Manifest
import android.bluetooth.*
import android.bluetooth.le.ScanCallback
import android.bluetooth.le.ScanFilter
import android.bluetooth.le.ScanResult
import android.bluetooth.le.ScanSettings
import android.content.Intent
import android.os.Bundle
import android.text.method.ScrollingMovementMethod
import android.util.Log
import android.widget.TextView
import kotlinx.android.synthetic.main.activity_main.*
import java.text.SimpleDateFormat
import java.util.*
import android.content.pm.PackageManager
import android.util.Base64
import android.view.Menu
import android.view.MenuItem
import android.view.View
import com.android.core.Utils
import com.ltdigital.sdk.android.demo.*
import com.ltdigital.sdk.android.demo.bean.VerifyResultBean
import com.ltdigital.sdk.android.demo.bluetooth.BleConstants
import com.ltdigital.sdk.android.demo.bluetooth.DeviceManager
import com.ltdigital.sdk.android.demo.bluetooth.Operation
import com.ltdigital.sdk.android.demo.callback.*
import com.ltdigital.sdk.android.demo.helper.NfcUtils
import com.ltdigital.sdk.android.demo.net.NetManager
import io.reactivex.rxjava3.functions.Consumer
import java.lang.StringBuilder
import kotlin.collections.ArrayList
import kotlin.math.min


class MainActivity : BaseActivity(), LogCallback, SpeedCallback, VerifyCallback {

    companion object {
        var bleManager: DeviceManager? = null
        val TAG = "MainActivity"
    }

    private val REQUEST_SELECT_DEVCIE = 1000
    private val REQUEST_DEVICE_TEST = 1001

    private var gatt: BluetoothGatt? = null
    private var curDevice: TextView? = null
    private var connectTargetMac: String? = null
    private var btManager: BluetoothManager? = null

    private val gattCallback = object : BluetoothGattCallback() {

        override fun onServicesDiscovered(gatt: BluetoothGatt?, status: Int) {
            Log.d("onServicesDiscovered", status.toString())
//            var mtu = 20
//            gatt?.requestMtu(mtu)
            onMtuChanged(gatt, 20, status)
        }

        override fun onMtuChanged(gatt: BluetoothGatt?, mtu: Int, status: Int) {
            super.onMtuChanged(gatt, mtu, status)
            gatt?.getService(BleConstants.DEVICE_DATA_SERVICE_UUID)?.let { service ->
                service.getCharacteristic(BleConstants.DEVICE_DATA_NOTIFY_CHARACTERISTIC_UUID)
                    ?.let {
                        printLog("MTU更新成功... mtu:$mtu")
                        bleManager?.refreshMtu(mtu)
                        enableNotifyCharacteristic(
                            gatt, it, BleConstants.DES_CLIENT_UUID
                        )
                    }
            } ?: let {
                printLog("MTU更新失败...自动断开连接")
                gatt?.disconnect()
            }
        }

        override fun onDescriptorWrite(
            gatt: BluetoothGatt?,
            descriptor: BluetoothGattDescriptor?,
            status: Int,
        ) {
            descriptor?.value?.let {
                if (Arrays.equals(
                        it, BluetoothGattDescriptor.ENABLE_NOTIFICATION_VALUE
                    ) && status != BluetoothGatt.GATT_SUCCESS
                ) {
                    //某个notify特征初始化失败，断开设备连接
                    Log.d(
                        TAG, "characteristic notify enable fail! ${descriptor.characteristic.uuid}"
                    )
                    gatt?.disconnect()
                    return
                }
            }

            descriptor?.characteristic?.service?.uuid?.let { serviceUUID ->
                if (serviceUUID == BleConstants.DEVICE_DATA_SERVICE_UUID) {
                    gatt?.getService(BleConstants.DEVICE_DATA_SERVICE_UUID)?.let { service ->
                        descriptor.characteristic?.uuid?.let {
                            gatt.setCharacteristicNotification(
                                service.getCharacteristic(it), true
                            )
                            if (it == BleConstants.DEVICE_FILE_DATETIME_NOTIFY_CHARACTERISTIC_UUID) {
                                //设备所有的notify服务已经初始化完
                                enableNotifyCharacteristic(
                                    gatt,
                                    gatt.getService(BleConstants.DEVICE_S_DATA_SERVICE_UUID)
                                        .getCharacteristic(BleConstants.DEVICE_S_DATA_NOTIFY_CHARACTERISTIC_UUID),
                                    BleConstants.DES_CLIENT_UUID
                                )
                            } else {
                                val uuid = when (it) {
                                    BleConstants.DEVICE_DATA_NOTIFY_CHARACTERISTIC_UUID -> BleConstants.DEVICE_FILE_NAME_NOTIFY_CHARACTERISTIC_UUID
                                    BleConstants.DEVICE_FILE_NAME_NOTIFY_CHARACTERISTIC_UUID -> BleConstants.DEVICE_STATUS_NOTIFY_CHARACTERISTIC_UUID
                                    BleConstants.DEVICE_STATUS_NOTIFY_CHARACTERISTIC_UUID -> BleConstants.DEVICE_REAL_TIME_NOTIFY_CHARACTERISTIC_UUID
                                    BleConstants.DEVICE_REAL_TIME_NOTIFY_CHARACTERISTIC_UUID -> BleConstants.DEVICE_CRC32_NOTIFY_CHARACTERISTIC_UUID
                                    BleConstants.DEVICE_CRC32_NOTIFY_CHARACTERISTIC_UUID -> BleConstants.DEVICE_STORAGE_INFO_NOTIFY_CHARACTERISTIC_UUID
                                    BleConstants.DEVICE_STORAGE_INFO_NOTIFY_CHARACTERISTIC_UUID -> BleConstants.DEVICE_FILE_DATETIME_NOTIFY_CHARACTERISTIC_UUID
                                    else -> null
                                }
                                uuid?.let {
                                    enableNotifyCharacteristic(
                                        gatt,
                                        service.getCharacteristic(it),
                                        BleConstants.DES_CLIENT_UUID
                                    )
                                }
                            }
                        }
                    }
                } else if (serviceUUID == BleConstants.DEVICE_S_DATA_SERVICE_UUID) {
                    gatt?.getService(BleConstants.DEVICE_S_DATA_SERVICE_UUID)?.let { service ->
                        descriptor.characteristic?.uuid?.let {
                            gatt.setCharacteristicNotification(
                                service.getCharacteristic(it), true
                            )
                            if (it == BleConstants.DEVICE_S_DATA_NOTIFY_CHARACTERISTIC_UUID) {
                                //设备所有的notify服务已经初始化完成
                                runOnUiThread {
                                    onBleDeviceInitSuccess()
                                }
                            }
                        }
                    }
                } else {

                }

            } ?: let {
                Log.d(
                    TAG, "characteristic notify enable fail! ${descriptor?.characteristic?.uuid}"
                )
                gatt?.disconnect()
            }


        }

        override fun onConnectionStateChange(gatt: BluetoothGatt?, status: Int, newState: Int) {
            Log.d("onConnectionStateChange", "$status-$newState")
            when (newState) {
                BluetoothGatt.STATE_CONNECTED -> {
                    printLog("设备已连接...准备初始化服务...")
                    //when gatt connected,then start discover services
                    runOnUiThread {
                        onBleConnected()
                    }
//                    gatt?.requestConnectionPriority(BluetoothGatt.CONNECTION_PRIORITY_HIGH)
//                    if (gatt != null && gatt.services.size > 0) {
//                        onServicesDiscovered(gatt, BluetoothGatt.GATT_SUCCESS)
//                        return
//                    }
                    val delay =
                        if (gatt?.device?.bondState == BluetoothDevice.BOND_BONDED) 1600L else 0
                    handler.postDelayed({
                        val discoverResult = gatt?.discoverServices()
                        Log.e(TAG, "discoverResult:$discoverResult")
                    }, delay)
                }
                BluetoothGatt.STATE_DISCONNECTED -> {
                    printLog("设备已断开连接... reason:$status")
                    gatt?.close()
                    runOnUiThread {
                        onBleDisconnected()
                    }
                }
            }
        }

        override fun onCharacteristicChanged(
            gatt: BluetoothGatt?,
            characteristic: BluetoothGattCharacteristic?,
        ) {
            characteristic?.let {
                bleManager?.onReceiveDeviceData(gatt, it, it.value)
            }
        }

        override fun onCharacteristicWrite(
            gatt: BluetoothGatt?,
            characteristic: BluetoothGattCharacteristic?,
            status: Int,
        ) {
            characteristic?.uuid?.let {
                if (status == 0) bleManager?.onCharacteristicWriteSuccess(it)
            }
        }
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        supportActionBar?.title = getString(R.string.app_name) + "_V" + BuildConfig.VERSION_NAME
        com.android.core.log.Log.init(cacheDir.absolutePath, true)
        setContentView(R.layout.activity_main)
        rxPermission.requestEachCombined(
            Manifest.permission.ACCESS_COARSE_LOCATION,
            Manifest.permission.ACCESS_FINE_LOCATION,
            Manifest.permission.BLUETOOTH,
            Manifest.permission.BLUETOOTH_ADMIN,
            Manifest.permission.WRITE_EXTERNAL_STORAGE,
            Manifest.permission.READ_EXTERNAL_STORAGE,
        ).subscribe(Consumer {
            if (it.granted.not()) {
                finish()
            }
        })
        btManager = getSystemService(BLUETOOTH_SERVICE) as BluetoothManager
        btn_connect.setOnClickListener {
            gatt?.disconnect() ?: let {
                gotoConnect()
            }
        }

        btn_check_code.setOnClickListener {
            startActivity(Intent(this, QRCodeScanActivity::class.java))
        }

        switch_verify.isChecked = MyPreferences.getPreferences(this).switchVerifyEnable
        switch_test.isChecked = MyPreferences.getPreferences(this).switchTestEnable

        switch_verify.setOnCheckedChangeListener { buttonView, isChecked ->
            MyPreferences.getPreferences(
                context
            ).switchVerifyEnable = isChecked
        }

        switch_test.setOnCheckedChangeListener { buttonView, isChecked ->
            MyPreferences.getPreferences(
                context
            ).switchTestEnable = isChecked
        }

        curDevice = findViewById(R.id.cur_device)

        tv_Log?.movementMethod = ScrollingMovementMethod.getInstance()
        initNfc()
    }

    override fun onCreateOptionsMenu(menu: Menu?): Boolean {
        menu?.add(1, 1, 1, "切换激活服务器")
        return true
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        if (item.order == 1) {
            //切换激活服务器
            startActivity(Intent(this, ChangeActiveServiceActivity::class.java))
        }
        return true
    }


    override fun onResume() {
        super.onResume()
        if (MyPreferences.getPreferences(this).activateCode == null || MyPreferences.getPreferences(
                this
            ).storageAppId == null
        ) {
            showToast("请先完成激活码填写")
            startActivity(Intent(this, QRCodeScanActivity::class.java))
            return
        }
        if (isSupportNfc()) {
            NfcUtils.mNfcAdapter?.enableForegroundDispatch(
                this, NfcUtils.mPendingIntent, NfcUtils.mIntentFilter, NfcUtils.mTechList
            )
        }
    }

    override fun onPause() {
        super.onPause()
        if (isSupportNfc()) {
            NfcUtils.mNfcAdapter?.disableForegroundDispatch(this)
        }
    }

    override fun onNewIntent(intent: Intent?) {
        super.onNewIntent(intent)
        val nfcContent: String? = NfcUtils.readNFCFromTag(intent)
        printLog("读到NFC标签内容：$nfcContent")
        var byteSize = 0
        byteSize = nfcContent?.length?.div(2) ?: 0
        nfcContent?.let {
            if (it.length < 13) {
                printLog("NFC标签内容不符合要求，解析失败")
                return
            }
            val macBuilder = StringBuilder()
            for (index in 0 until 6) {
                macBuilder.append(it.substring(index * 2, index * 2 + 2))
                if (index != 5) {
                    macBuilder.append(":")
                }
            }
            val mac = macBuilder.toString()
            if (BluetoothAdapter.checkBluetoothAddress(mac)) {
                printLog("NFC content:$mac")
                gatt?.let {
                    if (it.device.address != mac) {
                        it.disconnect()
                    } else {
                        showToast("设备已连接")
                        return
                    }
                }
                runOnUiThread {
                    dealDeviceSelect(mac)
                }
            } else {
                showToast("不是合法MAC地址")
            }
            if (it.length > 13) {
                //包含自定义内容
                val customStr = it.substring(13)
                printLog("NFC 标签自定义内容:$customStr")
            }
        }
    }

    private fun initNfc() {
        val nfcUtils = NfcUtils(this)
        printLog("当前设备是否支持NFC:${isSupportNfc()}")
    }

    private fun isSupportNfc(): Boolean {
        val packageManager = this.packageManager
        val b1 = packageManager.hasSystemFeature(PackageManager.FEATURE_NFC)
        return b1
    }

    private fun onBleDisconnected() {
        finishLoading()
        showToast("设备已断开连接")
        btn_connect.text = "连接设备"
        btn_connect.isEnabled = true
        bleManager?.release()
        bleManager?.setLogCallback(null)
        bleManager?.setSpeedCallback(null)
        bleManager?.setTestResultCallback(null)
        bleManager?.setVerifyCallback(null)
        bleManager = null
        connectTargetMac = null

        switch_verify?.isEnabled = true
        switch_test?.isEnabled = true

        vp_semi_auto.setCurrentItem(0, false)
        vp_semi_auto.visibility = View.INVISIBLE
        tv_desc.visibility = View.VISIBLE
        this@MainActivity.gatt = null
        curDevice?.text = "当前设备："
    }


    private fun onBleDeviceInitSuccess() {
        switch_verify?.isEnabled = false
        switch_test?.isEnabled = false
        finishLoading()
        printLog("设备初始化完成,可以开始通信...")
        btn_connect.text = "断开连接"
        btn_connect.isEnabled = true
        bleManager?.setLogCallback(this)
        bleManager?.setVerifyCallback(this)
        tv_desc.visibility = View.GONE
        if (switch_test.isChecked) {
            vp_semi_auto.visibility = View.VISIBLE
            bleManager?.sendTestCommand(arrayOf(0x01))
        }
        if (switch_test.isChecked) {
            startActivityForResult(
                Intent(this, DeviceTestActivity::class.java),
                REQUEST_DEVICE_TEST
            )
        } else if (switch_verify.isChecked) {
            beginVerify()
        }
    }

    private fun onBleConnected() {
        gatt?.let {
            bleManager = DeviceManager(applicationContext, it)
        }
    }


    private fun beginVerify() {
        bleManager?.sendCommand2(byteArrayOf(0x22, 0x01))
    }

    fun enableNotifyCharacteristic(
        gatt: BluetoothGatt, characteristic: BluetoothGattCharacteristic, descUUID: UUID
    ) {
        characteristic.getDescriptor(descUUID)?.let {
            it.value = BluetoothGattDescriptor.ENABLE_NOTIFICATION_VALUE
            gatt.writeDescriptor(it)
        }
    }


    private fun gotoConnect() {
        startActivityForResult(
            Intent(
                this, ConnectActivity::class.java
            ), REQUEST_SELECT_DEVCIE
        )
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        val success = resultCode == RESULT_OK
        when (requestCode) {
            REQUEST_SELECT_DEVCIE -> {
                if (success) {
                    val mac: String? = data?.getStringExtra("mac")
                    mac?.let {
                        dealDeviceSelect(it)
                    }
                }
            }
            REQUEST_DEVICE_TEST -> {
                if (success) {
                    //工厂测试通过
                    dealFactoryTestFinish()
                } else {
                    //工厂测试未通过
                    gatt?.disconnect()
                }
            }
        }
    }

    private fun dealDeviceSelect(mac: String) {
        //预防重复发起连接
        if (connectTargetMac != null) {
            printLog("当前正在连接设备 mac$connectTargetMac 请稍后重试")
            return
        }
        curDevice?.text = "当前设备：${mac}"
        connectDevice(mac)
    }

    private fun sendActiveCode(
        activeCode: ByteArray,
        appid: ByteArray,
        encryptData: ByteArray,
        sign: ByteArray
    ) {
        val sourceData = ArrayList<Byte>()
        sourceData.addAll(activeCode.asList())
        sourceData.addAll(appid.asList())
        sourceData.addAll(encryptData.asList())
        sourceData.addAll(sign.asList())
        val pkgDataLength = 17
        val pkgNum =
            sourceData.size / pkgDataLength + if (sourceData.size % pkgDataLength == 0) 0 else 1
        var sendArray = ArrayList<ByteArray>(pkgNum)
        for (pkgIndex in 0 until pkgNum) {
            val beginIndex = pkgIndex * pkgDataLength
            sendArray.add(
                sourceData.slice(beginIndex until min(beginIndex + pkgDataLength, sourceData.size))
                    .toByteArray()
            )
        }
        sendActiveCode(sendArray, 0)
    }

    private fun sendActiveCode(data: ArrayList<ByteArray>, pkgIndex: Int) {
        if (data.size == 0) return
        startLoading("正在发送激活信息，剩余:${data.size}")
        printLog("正在发送激活信息，当前剩余包数:${data.size}")
        val sourceData = data[0];
        val dataArray = ByteArray(sourceData.size + 3) {
            when (it) {
                0 -> 0x22
                1 -> 0x02
                2 -> pkgIndex.toByte()
                else -> sourceData[it - 3]
            }
        }
        bleManager?.sendCommand2(dataArray, object : Operation.DataSyncOperationCallback {
            override fun onFail(status: Int) {
                dealVerifyFail()
            }

            override fun onSuccess() {
                data.removeAt(0)
                if (data.size == 0) {
                    //最后一包发送完成
                    startLoading("激活信息发送完毕\n正在等待设备返回激活结果...")
                    printLog("激活信息发送完毕!")
                } else {
                    sendActiveCode(data, pkgIndex + 1)
                }
            }
        })
    }

    private fun connectDevice(mac: String?) {
        if (btManager?.adapter?.isEnabled == false) {
            btManager?.adapter?.enable()
            return
        }
        connectTargetMac = mac
        startLoading("正在连接设备...")
        printLog("开始连接设备... mac：$mac")
        btManager?.adapter?.let {
            it.bluetoothLeScanner.startScan(
                arrayListOf(ScanFilter.Builder().build()),
                ScanSettings.Builder().setScanMode(ScanSettings.SCAN_MODE_LOW_LATENCY).build(),
                scanCallback
            )
            handler.removeCallbacks(scanConnectTimeoutRunnable)
            handler.postDelayed(scanConnectTimeoutRunnable, 10 * 1000)
        }
    }

    val scanCallback = object : ScanCallback() {
        override fun onScanResult(callbackType: Int, result: ScanResult?) {
            if (result == null) return
            if (result.device == null) return
            if (result.device.address == null) return
            if (Objects.equals(result.device.address, connectTargetMac).not()) return
            dealDeviceScan(result.device)
        }
    }

    val scanConnectTimeoutRunnable = object : Runnable {
        override fun run() {
            handler.removeCallbacks(this)
            if (connectTargetMac == null) return
            btManager?.adapter?.bluetoothLeScanner?.stopScan(scanCallback)
            printLog("设备扫描超时，无法连接... mac：$connectTargetMac")
            finishLoading()
            connectTargetMac = null
        }
    }

    private fun dealDeviceScan(device: BluetoothDevice) {
        btn_connect.text = "正在连接..."
        btn_connect.isEnabled = false
        gatt = device.connectGatt(context, false, gattCallback)
        connectTargetMac = null
        handler.removeCallbacks(scanConnectTimeoutRunnable)
        btManager?.adapter?.bluetoothLeScanner?.stopScan(scanCallback)
    }

    private fun printLog(msg: String) {
        tv_Log?.post {
            tv_Log?.let {
                val format = SimpleDateFormat("HH:mm:ss.SSS", Locale.getDefault())
                it.append(
                    "${
                        format.format(Date(System.currentTimeMillis()))
                    } ->  $msg\n"
                )
                val offset =
                    it.lineCount * (it.lineHeight * it.lineSpacingMultiplier + it.lineSpacingExtra)
                val contentHeight = it.height - it.paddingTop - it.paddingBottom
                if (offset > contentHeight) {
                    it.scrollTo(0, (offset - contentHeight).toInt())
                }
            }
        }
    }

    /**
     * 工厂测试全部完成
     */
    private fun dealFactoryTestFinish() {
        showToast("测试完成")
        if (switch_verify.isChecked) {
            beginVerify()
        } else {
            gatt?.disconnect()
        }
    }

    override fun onLog(log: String) {
        printLog(log)
    }

    override fun onSpeedChange(kbPerSecond: Float) {
        printLog("实时传输速率:$kbPerSecond kb/s")
    }


    override fun onDeviceInfo(mac: String, deviceType: String, isVerify: Boolean) {
        printLog("mac:$mac , deviceType:$deviceType , activeSuccess:$isVerify")
//        if (isVerify) {
//            showToast("设备已激活，无需再次激活!")
//            gatt?.disconnect()
//            return
//        }
        val activateCode: String = MyPreferences.getPreferences(this).activateCode
        val appId: String = MyPreferences.getPreferences(this).storageAppId
        NetManager.getManager().getActivateCode(
            appId,
            activateCode,
            mac,
            deviceType,
            object : NetManager.CoreCallback(this) {
                override fun onData(p0: String?) {
                    try {
                        val verifyResult =
                            Utils.GSON.fromJson<VerifyResultBean>(p0, VerifyResultBean::class.java)
                        val encData = Base64.decode(verifyResult.finalEncData, Base64.DEFAULT)
                        val sign = Base64.decode(verifyResult.signedSha256, Base64.DEFAULT)
                        Log.e(TAG, "encData:${Arrays.toString(encData)}")
                        Log.e(TAG, "sign:${Arrays.toString(sign)}")
                        verifyResult?.let {
                            printLog(verifyResult.toString())
                            sendActiveCode(
                                activateCode.toByteArray(),
                                appId.toByteArray(),
                                encData,
                                sign
                            )
                        }
                    } catch (e: Exception) {
                        e.printStackTrace()
                        dealVerifyFail()
                        showToast("服务器返回解析结果有误！")
                    }
                }

                override fun onErrorData(code: Int, msg: String?) {
                    super.onErrorData(code, msg)
                    showToast(
                        when (code) {
                            4001 -> "参数错误"
                            4002 -> "AppId或secretKey不匹配"
                            4003 -> "secretKey与设备型号不匹配"
                            4006 -> "mac地址激活已到上限"
                            4004 -> "secretKey激活已到上限"
                            4005 -> "secretKey未激活"
                            else -> "未知错误"
                        }
                    )
                }
            })
    }

    private fun dealVerifyFail() {
        showToast("激活设备失败!")
        finishLoading()
        gatt?.disconnect()
    }

    /**
     * 设备返回的激活结果
     */
    override fun onVerifyResult(success: Boolean) {
        finishLoading()
        showToast("激活结果:${if (success) "成功" else "失败"}")
        gatt?.disconnect()
    }

    override fun showToast(p0: CharSequence?) {
        super.showToast(p0)
        p0?.let {
            printLog(it.toString())
        }
    }
}
