package com.ltdigital.sdk.android.demo.activity;

import android.Manifest;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.graphics.Bitmap;
import android.graphics.Canvas;
import android.graphics.ColorMatrix;
import android.graphics.ColorMatrixColorFilter;
import android.graphics.Paint;
import android.graphics.Point;
import android.os.Bundle;
import android.os.Handler;
import android.os.HandlerThread;
import android.os.Message;
import android.text.TextUtils;
import android.view.SurfaceHolder;
import android.widget.Button;
import android.widget.EditText;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.core.app.ActivityCompat;
import androidx.core.content.PermissionChecker;

import com.android.core.Utils;
import com.android.core.log.Log;
import com.google.gson.JsonSyntaxException;
import com.google.zxing.BarcodeFormat;
import com.google.zxing.Binarizer;
import com.google.zxing.BinaryBitmap;
import com.google.zxing.ChecksumException;
import com.google.zxing.DecodeHintType;
import com.google.zxing.FormatException;
import com.google.zxing.LuminanceSource;
import com.google.zxing.NotFoundException;
import com.google.zxing.PlanarYUVLuminanceSource;
import com.google.zxing.RGBLuminanceSource;
import com.google.zxing.Result;
import com.google.zxing.common.HybridBinarizer;
import com.google.zxing.qrcode.QRCodeReader;
import com.ltdigital.sdk.android.demo.Application;
import com.ltdigital.sdk.android.demo.MyPreferences;
import com.ltdigital.sdk.android.demo.R;
import com.ltdigital.sdk.android.demo.bean.QRCodeDecodeBean;
import com.ltdigital.sdk.android.demo.zxing.MySurfaceView;
import com.ltdigital.sdk.android.demo.zxing.camera.CameraManager;

import java.io.IOException;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Map;

import butterknife.BindView;
import butterknife.ButterKnife;

public class QRCodeScanActivity extends BaseQRCodeScanActivity implements SurfaceHolder.Callback, Handler.Callback {

    @BindView(R.id.preview_view)
    MySurfaceView previewView;
    @BindView(R.id.et_activating_code)
    EditText etActivatingCode;
    @BindView(R.id.et_appid)
    EditText etAppid;
    @BindView(R.id.bt_save)
    Button btSave;

    @Override
    protected void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        ButterKnife.bind(this);
    }

    @Override
    protected Integer getLayout() {
        return R.layout.activity_qr_code_scan;
    }

    @Override
    protected void initView() {
        super.initView();
        btSave.setOnClickListener(v -> {
            String activateCode = etActivatingCode.getText().toString().trim();
            String appId = etAppid.getText().toString().trim();
            if (TextUtils.isEmpty(activateCode)) {
                showToast("激活码不能为空");
                return;
            }
            if (TextUtils.isEmpty(appId)) {
                showToast("appId不能为空");
                return;
            }
            MyPreferences.getPreferences(this).setActivateCode(activateCode);
            MyPreferences.getPreferences(this).setStorageAppId(appId);
            showToast("激活码更新成功!");
            finish();
        });

    }

    @Override
    protected void initTitle() {
        this.isLightMode = false;
    }

    @Override
    protected void onResume() {
        super.onResume();
        Log.e(TAG, "onResume");
        if (etAppid != null) {
            etAppid.setText(MyPreferences.getPreferences(this).getStorageAppId());
        }
        if (etActivatingCode != null) {
            etActivatingCode.setText(MyPreferences.getPreferences(this).getActivateCode());
        }
    }

    /**
     * 二维码解析回调函数
     */
    public boolean dealDecodeResult(String result) {
        Log.d(TAG, "result:" + result);
        try {
            QRCodeDecodeBean decodeBean = Utils.GSON.fromJson(result, QRCodeDecodeBean.class);
            if (etAppid != null) {
                etAppid.setText(decodeBean.getAppId());
            }
            if (etActivatingCode != null) {
                etActivatingCode.setText(decodeBean.getSecretKey());
            }
            btSave.callOnClick();
            return true;
        } catch (JsonSyntaxException e) {
            e.printStackTrace();
            return false;
        }
    }

    @NonNull
    @Override
    MySurfaceView getSurfaceView() {
        return previewView;
    }
}
