package com.ltdigital.sdk.android.demo.fragment

import android.graphics.Color
import android.os.Bundle
import android.view.View
import com.android.core.base.fragment.BaseFragment
import com.ltdigital.sdk.android.demo.R
import com.ltdigital.sdk.android.demo.callback.TestControlCallback
import kotlinx.android.synthetic.main.fragment_test.*

/**
 * @author  Riven
 * @date  2021/11/9 14:55
 * @version 1.0
 */
class AutoResultFragment : BaseFragment() {

    var callback: TestControlCallback? = null
    var typeTag: String? = null
    var mTitle: String = ""
    var mContent: CharSequence = ""

    companion object {

        val TAG_TEST_TYPE_AUTO = "TAG_TEST_TYPE_AUTO"
        val TAG_TEST_TYPE_TOUCH = "TAG_TEST_TYPE_TOUCH"
        val TAG_TEST_TYPE_LED_VIBRATOR = "TAG_TEST_TYPE_LED_VIBRATOR"
        val TAG_TEST_TYPE_HEART_SPO2 = "TAG_TEST_TYPE_HEART_SPO2"
        val TAG_TEST_TYPE_CHARGER = "TAG_TEST_TYPE_CHARGER"
        val TAG_TEST_TYPE_POWER = "TAG_TEST_TYPE_POWER"

        fun newInstance(
            introduce: String,
            controlCallback: TestControlCallback,
            tag: String
        ): AutoResultFragment {
            val args = Bundle()
            args.putString("title", introduce)
            val fragment = AutoResultFragment()
            fragment.arguments = args
            fragment.callback = controlCallback
            fragment.typeTag = tag
            return fragment
        }
    }

    override fun getLayout(): Int {
        return R.layout.fragment_test
    }

    override fun initView(p0: View?) {

    }

    override fun onActivityCreated(savedInstanceState: Bundle?) {
        super.onActivityCreated(savedInstanceState)
        arguments?.let { bundle ->
            bundle.getString("title")?.let {
                mTitle = it
                tv_content.text = it
                tv_content.append("\n")
                tv_content.append(mContent)
            }
        }
        bt_pass?.setOnClickListener {
            callback?.onPass(typeTag)
        }

        bt_ng?.setOnClickListener {
            callback?.onNG(typeTag)
        }

        tv_next?.setOnClickListener {
            callback?.onNext(typeTag)
        }

        when (typeTag) {
            TAG_TEST_TYPE_AUTO -> {
                ll_result.visibility = View.GONE
                tv_next.visibility = View.VISIBLE
                ll_touch_result.visibility = View.GONE
            }
            TAG_TEST_TYPE_TOUCH -> {
                ll_result.visibility = View.GONE
                tv_next.visibility = View.GONE
                ll_touch_result.visibility = View.VISIBLE
            }
            else -> {
                ll_result.visibility = View.VISIBLE
                tv_next.visibility = View.GONE
                ll_touch_result.visibility = View.GONE
            }
        }

        if (typeTag == TAG_TEST_TYPE_CHARGER) {
            bt_pass.isEnabled = false
            bt_ng.isEnabled = false
        }
    }

    override fun onTimeChange(p0: Int) {

    }

    fun setContent(content: CharSequence) {
        mainHandler.post {
            mContent = content
            tv_content?.text = mTitle
            tv_content?.append("\n")
            tv_content?.append(content)
        }
    }

    fun setAutoTestIsPass(isPass: Boolean) {
        mainHandler.post {
            if (typeTag == TAG_TEST_TYPE_AUTO) {
                tv_next.isEnabled = isPass
            }
        }
    }

    fun reset() {
        mainHandler.post {
            tv_content?.text = mTitle
            if (typeTag == TAG_TEST_TYPE_CHARGER) {
                bt_pass.isEnabled = false
                bt_ng.isEnabled = false
            } else if (typeTag == TAG_TEST_TYPE_TOUCH) {
                val default = Color.parseColor("#666666")
                bt_left?.setBackgroundColor(default)
                bt_right?.setBackgroundColor(default)
            }
            setAutoTestIsPass(false)
        }
    }

    fun onTouchResult(
        left: Boolean,
        right: Boolean,
        leftLastPass: Boolean,
        rightLastPass: Boolean
    ) {
        mainHandler.post {
            val press = Color.parseColor("#0099FF")
            val pass = Color.parseColor("#33CC33")
            val default = Color.parseColor("#666666")
            bt_left?.setBackgroundColor(if (left) press else if (leftLastPass) pass else default)
            bt_right?.setBackgroundColor(if (right) press else if (rightLastPass) pass else default)
        }
    }

    fun onChargeSign() {
        mainHandler.post {
            setContent("检测到充电信号")
            bt_ng?.isEnabled = true
            bt_pass?.isEnabled = true
        }
    }

}