package com.ltdigital.sdk.android.demo.helper;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.lang.reflect.Array;
import java.util.HashMap;
import java.util.concurrent.Executor;
import java.util.concurrent.Executors;

public class FileSaveHelper {
    private File rootRawData;
    private HashMap<String, FileOutputStream> fileFosMaps;
    private HashMap<String, FileWriter> fileWriterMaps;
    private Executor executor;

    public FileSaveHelper(String dir, String path) throws FileNotFoundException {
        rootRawData = new File(dir, path);
        if (!rootRawData.exists())
            if (!rootRawData.mkdirs())
                throw new FileNotFoundException("could not create the dir--->" + rootRawData.getAbsolutePath());
        fileFosMaps = new HashMap<>();
        fileWriterMaps = new HashMap<>();
        executor = Executors.newSingleThreadExecutor();
    }

    public String getRawDataPath() {
        return rootRawData.getAbsolutePath();
    }

    public void saveStringLineData(final String file, final String data) {
        executor.execute(new Runnable() {
            @Override
            public void run() {
                FileWriter fos = getWriter(file);
                if (fos == null)
                    return;
                try {
                    fos.write(data + "\n");
                    fos.flush();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        });
    }


    public void saveByteData(final String file, final byte[] data) {
        executor.execute(new Runnable() {
            @Override
            public void run() {
                FileOutputStream fos = getFosWriter(file);
                if (fos == null)
                    return;
                try {
                    for (byte data : data) {
                        fos.write(data & 0xff);
                    }
                    fos.flush();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        });
    }


    public void close(final boolean hasRecord) {
        close(hasRecord, true);
    }

    public void close(final boolean hasRecord, final boolean upload) {
        executor.execute(new Runnable() {
            @Override
            public void run() {
                for (FileOutputStream fos : fileFosMaps.values()) {
                    if (fos != null) {
                        try {
                            fos.flush();
                            fos.close();
                        } catch (IOException e) {
                            e.printStackTrace();
                        }
                    }
                }
                fileFosMaps.clear();

                for (FileWriter fos : fileWriterMaps.values()) {
                    if (fos != null) {
                        try {
                            fos.flush();
                            fos.close();
                        } catch (IOException e) {
                            e.printStackTrace();
                        }
                    }
                }
                fileWriterMaps.clear();

                if (hasRecord) {
                    if (upload) {
                        // TODO: 2021/9/19 是否上传到服务器
                    }
                } else {
                    FileHelper.deleteFile(rootRawData.getAbsolutePath());
                }
            }
        });
    }

    public String getDataToString(Object data) {
        StringBuilder sb = new StringBuilder();
        if (data.getClass().isArray()) {
            int len = Array.getLength(data);
            for (int i = 0; i < len; i++) {
                sb.append(String.valueOf(Array.get(data, i)));
                if (i != len - 1) {
                    sb.append(",");
                }
            }
        } else {
            sb.append(String.valueOf(data));
        }
        return sb.toString();
    }

    private FileOutputStream getFosWriter(String file) {
        FileOutputStream fos = fileFosMaps.get(file);
        if (fos != null)
            return fos;
        try {
            fos = new FileOutputStream(new File(rootRawData, file), true);
            fileFosMaps.put(file, fos);
            return fos;
        } catch (IOException e) {
            return null;
        }
    }

    private FileWriter getWriter(String file) {
        FileWriter fos = fileWriterMaps.get(file);
        if (fos != null)
            return fos;
        try {
            fos = new FileWriter(new File(rootRawData, file), true);
            fileWriterMaps.put(file, fos);
            return fos;
        } catch (IOException e) {
            return null;
        }
    }

}
