package com.ltdigital.sdk.android.demo.helper

import android.app.Activity
import android.content.IntentFilter
import android.app.PendingIntent
import android.content.Intent
import com.ltdigital.sdk.android.demo.helper.NfcUtils
import android.content.IntentFilter.MalformedMimeTypeException
import android.nfc.*
import kotlin.Throws
import android.os.Parcelable
import android.nfc.tech.Ndef
import android.provider.Settings
import android.util.Log
import com.android.core.Utils
import java.io.IOException
import java.io.UnsupportedEncodingException
import java.nio.charset.StandardCharsets

/**
 * @author Riven
 * @version 1.0
 * @date 2021/9/27 9:12
 */
class NfcUtils(activity: Activity) {
    companion object {
        //nfc
        var mNfcAdapter: NfcAdapter? = null
        var mIntentFilter: Array<IntentFilter>? = null
        var mPendingIntent: PendingIntent? = null
        var mTechList: Array<Array<String>>? = null

        /**
         * 检查NFC是否打开
         */
        fun NfcCheck(activity: Activity): NfcAdapter? {
            val mNfcAdapter = NfcAdapter.getDefaultAdapter(activity)
            if (mNfcAdapter == null) {
                return null
            } else {
                if (!mNfcAdapter.isEnabled) {
                    val setNfc = Intent(Settings.ACTION_NFC_SETTINGS)
                    activity.startActivity(setNfc)
                }
            }
            return mNfcAdapter
        }

        /**
         * 初始化nfc设置
         */
        fun NfcInit(activity: Activity) {
            mPendingIntent = PendingIntent.getActivity(
                activity,
                0,
                Intent(activity, activity.javaClass).addFlags(Intent.FLAG_ACTIVITY_SINGLE_TOP),
                0
            )
            val filter = IntentFilter(NfcAdapter.ACTION_NDEF_DISCOVERED)
            val filter2 = IntentFilter(NfcAdapter.ACTION_TAG_DISCOVERED)
            try {
                filter.addDataType("*/*")
            } catch (e: MalformedMimeTypeException) {
                e.printStackTrace()
            }
            mIntentFilter = arrayOf(filter, filter2)
            mTechList = null
        }

        /**
         * 读取NFC的数据
         */
        @Throws(UnsupportedEncodingException::class)
        fun readNFCFromTag(intent: Intent?): String? {
            intent?.let {
                if (NfcAdapter.ACTION_NDEF_DISCOVERED == intent.action) {
                    val rawArray = intent.getParcelableArrayExtra(NfcAdapter.EXTRA_NDEF_MESSAGES)
                    if (rawArray != null) {
//                StringBuffer sbu = new StringBuffer();
//                for (int i = 0; i < rawArray.length; i++) {
//                    byte[] byteArray = ((NdefMessage) rawArray[i]).toByteArray();
//                    for (byte data : byteArray) {
//                        sbu.append((char) data);
//                    }
//                }
//                return sbu.toString();
                        val mNdefMsg = rawArray[0] as NdefMessage
                        val mNdefRecord = mNdefMsg.records[0]
                        if (mNdefRecord != null) {
                            Log.e(
                                "readNFCFromTag",
                                "getId:" + Utils.byteArrayToHexStrWithSpaceSplit(
                                    mNdefRecord.id
                                )
                            )
                            Log.e("readNFCFromTag", "tnf:" + mNdefRecord.tnf)
                            Log.e(
                                "readNFCFromTag",
                                "getType:" + Utils.byteArrayToHexStrWithSpaceSplit(
                                    mNdefRecord.type
                                )
                            )
                            return String(
                                mNdefRecord.payload.sliceArray(3 until mNdefRecord.payload.size),
                                StandardCharsets.UTF_8
                            )
                        }
                    }
                    return ""
                }
            }
            return null
        }

        /**
         * 往nfc写入数据
         */
        @Throws(IOException::class, FormatException::class)
        fun writeNFCToTag(data: String?, intent: Intent) {
            val tag = intent.getParcelableExtra<Tag>(NfcAdapter.EXTRA_TAG)
            val ndef = Ndef.get(tag)
            ndef.connect()
            val ndefRecord = NdefRecord.createTextRecord(null, data)
            val records = arrayOf(ndefRecord)
            val ndefMessage = NdefMessage(records)
            ndef.writeNdefMessage(ndefMessage)
        }

        /**
         * 读取nfcID
         */
        @Throws(UnsupportedEncodingException::class)
        fun readNFCId(intent: Intent): String {
            val tag =
                intent.getParcelableExtra<Tag>(NfcAdapter.EXTRA_TAG)
            return ByteArrayToHexString(tag!!.id)
        }

        /**
         * 将字节数组转换为字符串
         */
        private fun ByteArrayToHexString(inarray: ByteArray): String {
            var i: Int
            var j: Int
            var `in`: Int
            val hex = arrayOf(
                "0",
                "1",
                "2",
                "3",
                "4",
                "5",
                "6",
                "7",
                "8",
                "9",
                "A",
                "B",
                "C",
                "D",
                "E",
                "F"
            )
            var out = ""
            j = 0
            while (j < inarray.size) {
                `in` = inarray[j].toInt() and 0xff
                i = `in` shr 4 and 0x0f
                out += hex[i]
                i = `in` and 0x0f
                out += hex[i]
                ++j
            }
            return out
        }
    }

    /**
     * 构造函数，用于初始化nfc
     */
    init {
        mNfcAdapter = NfcCheck(activity)
        NfcInit(activity)
    }
}