package com.ltdigital.sdk.android.demo.net;

import android.text.TextUtils;

import androidx.annotation.NonNull;

import com.android.core.Utils;
import com.google.gson.annotations.SerializedName;

import java.lang.reflect.Type;

import okhttp3.Response;

public class BaseResult {

    @SerializedName("code")
    private Object status;

    @SerializedName("data")
    private Object data;

    @SerializedName("msg")
    private Object message;

    @SerializedName("success")
    private Object success;

    public static BaseResult parseFromJson(String json) {
        return Utils.GSON.fromJson(json, BaseResult.class);
    }

    public static BaseResult parseFromResponse(Response response) {
        if (response == null || !response.isSuccessful() || response.body() == null) {
            BaseResult result = new BaseResult();
            result.status = -1;
            result.message = "response is not success";
            result.success = false;
            return result;
        }
        String contentType = response.headers().get("Content-Type");
        if (TextUtils.isEmpty(contentType) || !contentType.toLowerCase().contains("json")) {
            BaseResult result = new BaseResult();
            result.status = -3;
            result.message = "content type is not json";
            result.success = false;
            return result;
        }
        try {
            return Utils.GSON.fromJson(response.body().string(), BaseResult.class);
        } catch (Exception e) {
            e.printStackTrace();
            BaseResult result = new BaseResult();
            result.status = -1;
            result.message = "response is not success";
            result.success = false;
            return result;
        }
    }

    public int getIntResultCode() {
        if (status == null)
            return -1;
        try {
            return (int) Float.parseFloat(status.toString());
        } catch (NumberFormatException e) {
            return -1;
        }
    }

    public String getStringResultMsg() {
        if (message == null)
            return "";
        return message.toString();
    }

    public <T> T getResultDataFromType(@NonNull Type t) {
        if (data == null) {
            return null;
        }
        try {
            return Utils.GSON.fromJson(getResultDataToJsonString(), t);
        } catch (Exception e) {
            e.printStackTrace(System.out);
            return null;
        }
    }

    public <T> T getResultDataFromClass(@NonNull Class<T> tClass) {
        if (data == null) {
            return null;
        }
        try {
            return Utils.GSON.fromJson(getResultDataToJsonString(), tClass);
        } catch (Exception e) {
            return null;
        }
    }

    public String getResultDataToJsonString() {
        if (data == null) {
            return "";
        }
        return Utils.GSON.toJson(data);
    }

    public Object getStatus() {
        return status;
    }

    public void setStatus(Object status) {
        this.status = status;
    }

    public Object getData() {
        return data;
    }

    public void setData(Object data) {
        this.data = data;
    }

    public Object getMessage() {
        return message;
    }

    public void setMessage(Object message) {
        this.message = message;
    }

    public Object getSuccess() {
        return success;
    }

    public void setSuccess(Object success) {
        this.success = success;
    }

    @Override
    public String toString() {
        return "BaseResult{" +
                "status=" + status +
                ", data=" + data +
                ", message=" + message +
                ", success=" + success +
                '}';
    }
}
