package com.ltdigital.sdk.android.demo.net;

import android.os.Trace;

import com.android.core.Utils;
import com.android.core.base.mvp.inter.IView;
import com.android.core.log.Log;
import com.ltdigital.sdk.android.demo.Application;
import com.ltdigital.sdk.android.demo.MyPreferences;

import java.util.HashMap;


/**
 * @author Riven
 * @version 1.0
 * @date 2020/12/9 15:51
 */
public class NetManager {
    private static NetManager manager;

    public static NetManager getManager() {
        if (manager == null) {
            manager = new NetManager();
        }
        return manager;
    }

    public void getActivateCode(String appId, String secretKey, String deviceMac, String deviceType, CoreCallback callback) {
        HashMap<String, String> jsonMap = new HashMap<>();
        jsonMap.put("appId", appId);
        jsonMap.put("secretKey", secretKey);
        jsonMap.put("deviceMac", deviceMac);
        jsonMap.put("deviceType", deviceType);

        MyOkHttpHelper.getHelper().postJson()
                .json(Utils.GSON.toJson(jsonMap))
                .tag(callback.view)
                .url(MyPreferences.getPreferences(Application.getAppContext()).getStorageActiveServerUrl() + "/device/authentication")
                .build().call().execute(callback);
    }

    public abstract static class CoreCallback extends BaseLoadableCallback {

        public CoreCallback(IView view) {
            super(view);
        }

        @Override
        public void onResponse(String s) {
            Trace.beginSection("response start");
            try {
                BaseResult result = BaseResult.parseFromJson(s);
                int resultCode = result.getIntResultCode();
                String msg = result.getStringResultMsg();
                switch (resultCode) {
                    case 200:
                        Trace.beginSection("response data start");
                        this.onData(result.getResultDataToJsonString());
                        Trace.endSection();
                        break;
                    default:
                        Trace.beginSection("response error start");
                        this.onErrorData(resultCode, msg);
                        Trace.endSection();
                }
            } catch (Exception var5) {
                Trace.beginSection("response exception start");
                Log.e(android.util.Log.getStackTraceString(var5));
                this.onErrorData(65533, this.getDataDecodeErrorContent());
                Trace.endSection();
            }
            Trace.endSection();
        }

        @Override
        public void onErrorData(int code, String msg) {
            if (code < 7000) {
                if (view != null) {
                    view.showToast(msg);
                }
            } else {
                if (view != null) {
                    view.showToast("网络请求超时");
                }
                Log.log(msg);
            }
        }
    }
}
