package com.ltdigital.sdk.android.demo.zxing;

import com.google.zxing.Binarizer;
import com.google.zxing.LuminanceSource;
import com.google.zxing.NotFoundException;
import com.google.zxing.common.BitMatrix;
import com.google.zxing.common.GlobalHistogramBinarizer;

/**
 * 2021-11-03
 * @author dpeng
 */
public final class OtsuBinarizer extends GlobalHistogramBinarizer {

    private BitMatrix matrix;

    public OtsuBinarizer(LuminanceSource source) {
        super(source);
    }

    @Override
    public BitMatrix getBlackMatrix() throws NotFoundException {
        if (matrix != null) {
            return matrix;
        }
        LuminanceSource source = getLuminanceSource();
        int width = source.getWidth();
        int height = source.getHeight();
        byte[] luminances = source.getMatrix();
        int length = width * height;
        int[] histogram = getHistogram(luminances, length);
        int threshold = getThreshold(histogram, length);
        matrix = new BitMatrix(width, height);
        for (int i = 0; i < height; i++) {
            for (int j = 0; j < width; j++) {
                int pixel = luminances[i * width + j] & 0xFF;
                if (pixel <= threshold) matrix.set(j, i);
            }
        }
        return matrix;
    }

    @Override
    public Binarizer createBinarizer(LuminanceSource source) {
        return new OtsuBinarizer(source);
    }

    private int getThreshold(int[] histogram, int total) {
        double except = 0, uT = 0, bestVar = 0;
        int acc = 0, bestValue = 0;
        for (int i = 0; i < 256; i++) {
            uT += i * histogram[i];
        }
        for (int t = 0; t < 255; t++) {
            acc += histogram[t];
            except += histogram[t] * t;
            if (acc == 0) {
                continue;
            }
            if (total == acc) {
                break;
            }
            double u0 = except / acc;
            double u1 = (uT - except) / (total - acc);
            double var = (u0 - u1) * (u0 - u1) * acc * (total - acc);
            if (var > bestVar) {
                bestVar = var;
                bestValue = t;
            }
        }
        return bestValue;
    }

    /**
     * Get the histogram of a image
     *
     * @param pixels pixel array of the image
     * @return the histogram of the image
     */
    private static int[] getHistogram(byte[] pixels, int length) {
        int[] histogram = new int[256];
        for (int i = 0; i < length; i++) {
            ++histogram[pixels[i] & 0xFF];
        }
        return histogram;
    }
}
